/*
    libtree : C++ lib that helps to work with tree-like data structures
    Copyright (C) 2019-2021 Alexey Appolonov

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef     ltree_selected_values_of_additional_branchesH
#define     ltree_selected_values_of_additional_branchesH

#include    "main.h"
#include    <map>
#include    <functional>

namespace tree{

//-----------------------------------------------------------------------------
// Tagged values - a vector of pairs {value, tag} in which a tag tells some
// additional information about a value of this pair (for example, where this
// value came from)

class TTaggedValues : public vector<pair<string, int>>{

    private:

        bool unit_opened = false;

    public:

        void PushListEl(const string &str);
        void CloseList(const int tag=-1);

        void push_back(const string &str, const int tag=-1){
            unit_opened = false;
            this->vector<pair<string, int>>::push_back({str, tag});
            }

        void clear(void){
            unit_opened = false;
            this->vector<pair<string, int>>::clear();
            }
    };

//-----------------------------------------------------------------------------
// Selected values found at specified levels (third key) of a tree-like
// document, that (values) come from specified additional branches (second key)
// of specified master branches (first key)

class TSelectedValues : public map<int, map<int, map<int, TTaggedValues>>>{

    public:

        map<int, vector<string>> Align(const int master_branch,
                function<bool(int)> condition=[](int cell){return true;}) const;

        #ifdef LTREE_DEBUGGING
        void Print(void) const;
        #endif
    };

//-----------------------------------------------------------------------------

} // namespace tree

#endif
